@@ -1,5 +1,5 @@
 from . import collection, model, rdf, shared
-from .commands import validate
+from .commands import update_format, update_rdf, validate
 from .io_ import (
     get_resource_package_content,
     load_raw_resource_description,
