@@ -13,6 +13,7 @@ from types import ModuleType
 from typing import Dict, IO, Optional, Sequence, Tuple, Union
 
 from marshmallow import missing
+from packaging.version import Version
 
 from bioimageio.spec.shared import RDF_NAMES, raw_nodes, resolve_rdf_source, resolve_rdf_source_and_type, resolve_source
 from bioimageio.spec.shared.common import (
@@ -26,10 +27,11 @@ from bioimageio.spec.shared.common import (
     yaml,
 )
 from bioimageio.spec.shared.node_transformer import (
+    AbsoluteToRelativePathTransformer,
     GenericRawNode,
     GenericRawRD,
-    PathToRemoteUriTransformer,
     RawNodePackageTransformer,
+    RelativePathTransformer,
 )
 from bioimageio.spec.shared.raw_nodes import ResourceDescription as RawResourceDescription
 from bioimageio.spec.shared.schema import SharedBioImageIOSchema
@@ -124,27 +126,6 @@ def extract_resource_package(
     return source, source_name, package_path
 
 
-def _replace_relative_paths_for_remote_source(raw_rd: RawResourceDescription) -> RawResourceDescription:
-    root = raw_rd.root_path
-    if isinstance(root, raw_nodes.URI):
-        # for a remote source relative paths are invalid; replace all relative file paths in source with URLs
-        raw_rd = PathToRemoteUriTransformer(remote_source=root).transform(raw_rd)
-        root_path = pathlib.Path()  # root_path cannot be URI
-    elif isinstance(root, pathlib.Path):
-        if zipfile.is_zipfile(root):
-            _, _, root_path = extract_resource_package(root)
-        else:
-            root_path = root
-    elif isinstance(root, bytes):
-        raise NotImplementedError("root as bytes (io)")
-    else:
-        raise TypeError(root)
-
-    assert isinstance(root_path, pathlib.Path)
-    raw_rd.root_path = root_path.resolve()
-    return raw_rd
-
-
 def load_raw_resource_description(
     source: Union[dict, os.PathLike, IO, str, bytes, raw_nodes.URI, RawResourceDescription],
     update_to_format: Optional[str] = None,
@@ -178,8 +159,18 @@ def load_raw_resource_description(
 
     class_name = get_class_name_from_type(type_)
 
+    # determine submodule's format version
+    original_data_version = data.get("format_version")
+    if original_data_version is None:
+        odv: Optional[Version] = None
+    else:
+        try:
+            odv = Version(original_data_version)
+        except Exception as e:
+            raise ValueError(f"Invalid format version {original_data_version}.") from e
+
     if update_to_format is None:
-        data_version = data.get("format_version", LATEST)
+        data_version = original_data_version or LATEST
     elif update_to_format == LATEST:
         data_version = LATEST
     else:
@@ -190,24 +181,73 @@ def load_raw_resource_description(
                 f"(always updating to latest patch version)."
             )
 
-    sub_spec = _get_spec_submodule(type_, data_version)
+    try:
+        sub_spec = _get_spec_submodule(type_, data_version)
+    except ValueError as e:
+        if odv is None:
+            raise e  # raise original error; no second attempt with 'LATEST' format version
+
+        try:
+            # load latest spec submodule
+            sub_spec = _get_spec_submodule(type_, data_version=LATEST)
+        except ValueError:
+            raise e  # raise original error with desired data_version
+
+        if odv <= Version(sub_spec.format_version):
+            # original format version is not a future version.
+            # => we should not fall back to latest format version.
+            # => 'format_version' may be invalid or the issue lies with 'type_'...
+            raise e
+
+    if odv and Version(sub_spec.format_version) < odv:
+        warnings.warn(
+            f"Loading future {type_} format version {original_data_version} as (latest known) "
+            f"{sub_spec.format_version}."
+        )
+        data["format_version"] = sub_spec.format_version  # set format_version to latest available
+
+        # save original format version under config:bioimageio:original_format_version for reference
+        if "config" not in data:
+            data["config"] = {}
+
+        if "bioimageio" not in data["config"]:
+            data["config"]["bioimageio"] = {}
+
+        data["config"]["bioimageio"]["original_format_version"] = original_data_version
+
     schema: SharedBioImageIOSchema = getattr(sub_spec.schema, class_name)()
 
     data = sub_spec.converters.maybe_convert(data)
     raw_rd = schema.load(data)
-    raw_rd.root_path = root
 
-    raw_rd = _replace_relative_paths_for_remote_source(raw_rd)
+    if isinstance(root, pathlib.Path):
+        root = root.resolve()
+        if zipfile.is_zipfile(root):
+            # set root to extracted zip package
+            _, _, root = extract_resource_package(root)
+    elif isinstance(root, bytes):
+        root = pathlib.Path().resolve()
+
+    raw_rd.root_path = root
+    raw_rd = RelativePathTransformer(root=root).transform(raw_rd)
 
     return raw_rd
 
 
-def serialize_raw_resource_description_to_dict(raw_rd: RawResourceDescription) -> dict:
-    """serialize a raw nodes resource description to a dict with the content of a resource description file (RDF)"""
+def serialize_raw_resource_description_to_dict(
+    raw_rd: RawResourceDescription, convert_absolute_paths: bool = False
+) -> dict:
+    """serialize a raw nodes resource description to a dict with the content of a resource description file (RDF).
+    If 'convert_absolute_paths' all absolute paths are converted to paths relative to raw_rd.root_path before
+    serialization.
+    """
     class_name = get_class_name_from_type(raw_rd.type)
     sub_spec = _get_spec_submodule(raw_rd.type, raw_rd.format_version)
-
     schema: SharedBioImageIOSchema = getattr(sub_spec.schema, class_name)()
+
+    if convert_absolute_paths:
+        raw_rd = AbsoluteToRelativePathTransformer(root=raw_rd.root_path).transform(raw_rd)
+
     serialized = schema.dump(raw_rd)
     assert isinstance(serialized, dict)
     assert missing not in serialized.values()
@@ -215,11 +255,11 @@ def serialize_raw_resource_description_to_dict(raw_rd: RawResourceDescription) -
     return serialized
 
 
-def serialize_raw_resource_description(raw_rd: RawResourceDescription) -> str:
+def serialize_raw_resource_description(raw_rd: RawResourceDescription, convert_absolute_paths: bool = True) -> str:
     if yaml is None:
         raise RuntimeError("'serialize_raw_resource_description' requires yaml")
 
-    serialized = serialize_raw_resource_description_to_dict(raw_rd)
+    serialized = serialize_raw_resource_description_to_dict(raw_rd, convert_absolute_paths=convert_absolute_paths)
 
     with StringIO() as stream:
         yaml.dump(serialized, stream)
