@@ -3,6 +3,8 @@ from typing import Any, Dict
 
 from marshmallow import missing
 
+from bioimageio.spec.rdf.v0_2.converters import remove_slash_from_names
+
 
 def convert_model_from_v0_3_to_0_4_0(data: Dict[str, Any]) -> Dict[str, Any]:
     from bioimageio.spec.model import v0_3
@@ -64,6 +66,15 @@ def convert_model_from_v0_4_4_to_0_4_5(data: Dict[str, Any]) -> Dict[str, Any]:
     return data
 
 
+def convert_model_from_v0_4_6_to_0_4_7(data: Dict[str, Any]) -> Dict[str, Any]:
+    data = dict(data)
+
+    remove_slash_from_names(data)
+
+    data["format_version"] = "0.4.7"
+    return data
+
+
 def maybe_convert(data: Dict[str, Any]) -> Dict[str, Any]:
     """auto converts model 'data' to newest format"""
     major, minor, patch = map(int, data.get("format_version", "0.3.0").split("."))
@@ -79,6 +90,12 @@ def maybe_convert(data: Dict[str, Any]) -> Dict[str, Any]:
     if data["format_version"] == "0.4.4":
         data = convert_model_from_v0_4_4_to_0_4_5(data)
 
+    if data["format_version"] == "0.4.5":
+        data["format_version"] = "0.4.6"
+
+    if data["format_version"] == "0.4.6":
+        data = convert_model_from_v0_4_6_to_0_4_7(data)
+
     # remove 'future' from config if no other than the used future entries exist
     config = data.get("config", {})
     if config.get("future") == {}:
