@@ -5,6 +5,8 @@ serialization and deserialization are defined in schema:
 RDF <--schema--> raw nodes
 """
 import dataclasses
+import pathlib
+
 import packaging.version
 import warnings
 from dataclasses import dataclass
@@ -22,7 +24,7 @@ except ImportError:
     from typing_extensions import Literal, get_args  # type: ignore
 
 FormatVersion = Literal[
-    "0.2.0", "0.2.1", "0.2.2"
+    "0.2.0", "0.2.1", "0.2.2", "0.2.3"
 ]  # newest format needs to be last (used to determine latest format version)
 
 
@@ -88,7 +90,7 @@ class RDF(ResourceDescription):
     covers: Union[_Missing, List[Union[URI, Path]]] = missing
     description: str = missing
     documentation: Union[_Missing, Path, URI] = missing
-    download_url: Union[_Missing, URI] = missing
+    download_url: Union[_Missing, Path, URI] = missing
     format_version: FormatVersion = missing
     git_repo: Union[_Missing, str] = missing
     id: Union[_Missing, str] = missing
@@ -109,6 +111,7 @@ class RDF(ResourceDescription):
         name: str,
         type: str = missing,
         version: Union[_Missing, packaging.version.Version] = missing,
+        root_path: pathlib.Path = pathlib.Path(),
         # RDF
         attachments: Union[_Missing, Dict[str, Any]] = missing,
         authors: Union[_Missing, List[Author]] = missing,
@@ -118,7 +121,7 @@ class RDF(ResourceDescription):
         covers: Union[_Missing, List[Union[URI, Path]]] = missing,
         description: str,
         documentation: Union[_Missing, Path, URI] = missing,
-        download_url: Union[_Missing, URI] = missing,
+        download_url: Union[_Missing, Path, URI] = missing,
         git_repo: Union[_Missing, str] = missing,
         id: Union[_Missing, str] = missing,
         icon: Union[_Missing, str] = missing,
@@ -148,12 +151,11 @@ class RDF(ResourceDescription):
         self.rdf_source = rdf_source
         self.source = source
         self.tags = tags
-        super().__init__(format_version=format_version, name=name, type=type, version=version)
+        super().__init__(format_version=format_version, name=name, type=type, version=version, root_path=root_path)
 
         if unknown_kwargs:
             # make sure we didn't forget a defined field
             field_names = set(f.name for f in dataclasses.fields(self))
-            field_names.remove("root_path")  # ignore internal root_path field
             for uk in unknown_kwargs:
                 assert uk not in field_names, uk
 
