@@ -20,13 +20,13 @@ class _BioImageIOSchema(SharedBioImageIOSchema):
 
 class Attachments(_BioImageIOSchema, WithUnknown):
     files = fields.List(
-        fields.Union([fields.URI(), fields.RelativeLocalPath()]),
+        fields.Union([fields.URI(), fields.Path()]),
         bioimageio_description="File attachments; included when packaging the resource.",
     )
 
 
 class _Person(_BioImageIOSchema):
-    name = fields.String(bioimageio_description="Full name.")
+    name = fields.Name(bioimageio_description="Full name.")
     affiliation = fields.String(bioimageio_description="Affiliation.")
     email = fields.Email()
     github_user = fields.String(bioimageio_description="GitHub user name.")  # todo: add validation?
@@ -44,7 +44,7 @@ class _Person(_BioImageIOSchema):
 
 
 class Author(_Person):
-    name = fields.String(require=True, bioimageio_description="Full name.")
+    name = fields.Name(require=True, bioimageio_description="Full name.")
 
 
 class Maintainer(_Person):
@@ -56,7 +56,7 @@ class Badge(_BioImageIOSchema):
     label = fields.String(required=True, bioimageio_description="e.g. 'Open in Colab'")
     icon = fields.String(bioimageio_description="e.g. 'https://colab.research.google.com/assets/colab-badge.svg'")
     url = fields.Union(
-        [fields.URL(), fields.RelativeLocalPath()],
+        [fields.URL(), fields.Path()],
         bioimageio_description="e.g. 'https://colab.research.google.com/github/HenriquesLab/ZeroCostDL4Mic/blob/master/Colab_notebooks/U-net_2D_ZeroCostDL4Mic.ipynb'",
     )
 
@@ -126,7 +126,7 @@ E.g. the citation for the model architecture and/or the training data used."""
     config = fields.YamlDict(bioimageio_descriptio=config_bioimageio_description)
 
     covers = fields.List(
-        fields.Union([fields.URL(), fields.RelativeLocalPath()]),
+        fields.Union([fields.URL(), fields.Path()]),
         bioimageio_description="A list of cover images provided by either a relative path to the model folder, or a "
         "hyperlink starting with 'http[s]'. Please use an image smaller than 500KB and an aspect ratio width to height "
         "of 2:1. The supported image formats are: 'jpg', 'png', 'gif'.",  # todo: field_validators image format
@@ -137,7 +137,7 @@ E.g. the citation for the model architecture and/or the training data used."""
     documentation = fields.Union(
         [
             fields.URL(),
-            fields.RelativeLocalPath(
+            fields.Path(
                 validate=field_validators.Attribute(
                     "suffix",
                     field_validators.Equal(
@@ -150,7 +150,9 @@ E.g. the citation for the model architecture and/or the training data used."""
         "For markdown files the recommended documentation file name is `README.md`.",
     )
 
-    download_url = fields.URL(bioimageio_description="optional url to download the resource from")
+    download_url = fields.Union(
+        [fields.URL(), fields.Path()], bioimageio_description="optional url to download the resource from"
+    )
 
     format_version = fields.String(
         required=True,
@@ -209,7 +211,7 @@ E.g. the citation for the model architecture and/or the training data used."""
 
     maintainers = fields.List(fields.Nested(Maintainer()), bioimageio_description="Maintainers of this resource.")
 
-    name = fields.String(required=True, bioimageio_description="name of the resource, a human-friendly name")
+    name = fields.Name(required=True, bioimageio_description="name of the resource, a human-friendly name")
 
     @validates
     def warn_about_long_name(self, value: str):
@@ -225,7 +227,7 @@ E.g. the citation for the model architecture and/or the training data used."""
         [fields.URL(), fields.DOI()], bioimageio_description="url or doi to the source of the resource definition"
     )
     source = fields.Union(
-        [fields.URI(), fields.RelativeLocalPath()],
+        [fields.URI(), fields.Path()],
         bioimageio_description="url or local relative path to the source of the resource",
     )
 
@@ -263,7 +265,7 @@ E.g. the citation for the model architecture and/or the training data used."""
         if value != schema_type:
             self.warn("type", f"Unrecognized type '{value}'. Validating as {schema_type}.")
 
-    version = fields.StrictVersion(
+    version = fields.Version(
         bioimageio_description="The version number of the model. The version number format must be a string in "
         "`MAJOR.MINOR.PATCH` format following the guidelines in Semantic Versioning 2.0.0 (see https://semver.org/), "
         "e.g. the initial version number should be `0.1.0`."
