@@ -9,6 +9,7 @@ from ._resolve_source import (
     BIOIMAGEIO_SITE_CONFIG,
     BIOIMAGEIO_SITE_CONFIG_ERROR,
     RDF_NAMES,
+    DownloadCancelled,
     _resolve_json_from_url,
     resolve_local_source,
     resolve_rdf_source,
@@ -16,6 +17,7 @@ from ._resolve_source import (
     resolve_source,
     source_available,
 )
+from ._update_nested import update_nested
 from .common import get_args, yaml  # noqa
 
 _license_file = Path(__file__).parent.parent / "static" / "licenses.json"
