@@ -1,13 +1,14 @@
+import getpass
 import os
 import pathlib
 import tempfile
 import warnings
-from typing import Generic, Optional, Sequence
+from typing import Any, Dict, Generic, Iterable, List, Optional, Sequence, Union
 
 try:
-    from typing import Literal, get_args, get_origin, Protocol
+    from typing import Literal, get_args, get_origin, Protocol, TypedDict
 except ImportError:
-    from typing_extensions import Literal, get_args, get_origin, Protocol  # type: ignore
+    from typing_extensions import Literal, get_args, get_origin, Protocol, TypedDict  # type: ignore
 
 
 try:
@@ -33,9 +34,39 @@ else:
     yaml = MyYAML(typ="safe")
 
 
+try:
+    from tqdm import tqdm  # not available in pyodide
+except ImportError:
+
+    class tqdm:  # type: ignore
+        """no-op tqdm"""
+
+        def __init__(self, iterable: Optional[Iterable] = None, *args, **kwargs):
+            self.iterable = iterable
+
+        def __iter__(self):
+            if self.iterable is not None:
+                yield from self.iterable
+
+        def update(self, *args, **kwargs):
+            pass
+
+
+class CacheWarning(RuntimeWarning):
+    pass
+
+
 BIOIMAGEIO_CACHE_PATH = pathlib.Path(
-    os.getenv("BIOIMAGEIO_CACHE_PATH", pathlib.Path(tempfile.gettempdir()) / "bioimageio_cache")
+    os.getenv("BIOIMAGEIO_CACHE_PATH", pathlib.Path(tempfile.gettempdir()) / getpass.getuser() / "bioimageio_cache")
 )
+BIOIMAGEIO_USE_CACHE = os.getenv("BIOIMAGEIO_USE_CACHE", "true").lower() in ("true", "yes", "1")
+BIOIMAGEIO_CACHE_WARNINGS_LIMIT = int(os.getenv("BIOIMAGEIO_CACHE_WARNINGS_LIMIT", 3))
+
+# keep a reference to temporary directories and files.
+# These temporary locations are used instead of paths in BIOIMAGEIO_CACHE_PATH if BIOIMAGEIO_USE_CACHE is true,
+# so that we guarantee that these temporary directories/file objects are not garbage collected too early
+# and thus their content removed from disk, while we still have a pathlib.Path pointing there
+no_cache_tmp_list: List[Any] = []
 
 BIOIMAGEIO_SITE_CONFIG_URL = "https://raw.githubusercontent.com/bioimage-io/bioimage.io/main/site.config.json"
 BIOIMAGEIO_COLLECTION_URL = "https://bioimage-io.github.io/collection-bioimage-io/collection.json"
@@ -49,10 +80,10 @@ class ValidationWarning(UserWarning):
     """a warning category to warn with during RDF validation"""
 
     @staticmethod
-    def get_warning_summary(val_warns: Sequence[warnings.WarningMessage]) -> dict:
+    def get_warning_summary(val_warns: Optional[Sequence[warnings.WarningMessage]]) -> dict:
         """Summarize warning messages of the ValidationWarning category"""
 
-        def add_to_summary(s, keys, msg):
+        def add_val_warn_to_summary(s, keys, msg):
             key = keys.pop(0)
             if "[" in key:
                 key, rest = key.split("[")
@@ -74,24 +105,41 @@ class ValidationWarning(UserWarning):
 
             if keys:
                 assert isinstance(s, dict), (keys, s)
-                add_to_summary(s, keys, msg)
+                add_val_warn_to_summary(s, keys, msg)
 
         summary: dict = {}
-        for vw in val_warns:
-            assert issubclass(vw.category, ValidationWarning)
+        nvw: set = set()
+        for vw in val_warns or []:
             msg = str(vw.message)
-            if ": " in msg:
-                keys_, *rest = msg.split(": ")
-                msg = ": ".join(rest)
-                keys = keys_.split(":")
-            else:
-                keys = []
+            if issubclass(vw.category, ValidationWarning):
+                if ": " in msg:
+                    keys_, *rest = msg.split(": ")
+                    msg = ": ".join(rest)
+                    keys = keys_.split(":")
+                else:
+                    keys = []
 
-            add_to_summary(summary, keys, msg)
+                add_val_warn_to_summary(summary, keys, msg)
+            else:
+                nvw.add(msg)
+
+        if nvw:
+            summary["non-validation-warnings"] = list(nvw)
 
         return summary
 
 
+class ValidationSummary(TypedDict):
+    bioimageio_spec_version: str
+    error: Union[None, str, Dict[str, Any]]
+    name: str
+    nested_errors: Dict[str, dict]  # todo: mark as not required: typing_extensions.NotRequired (typing py 3.11)
+    source_name: str
+    status: Union[Literal["passed", "failed"], str]
+    traceback: Optional[List[str]]
+    warnings: dict
+
+
 def get_format_version_module(type_: str, format_version: str):
     assert "." in format_version
     import bioimageio.spec
@@ -113,8 +161,8 @@ def get_patched_format_version(type_: str, format_version: str):
     return version_mod.format_version
 
 
-def get_spec_type_from_type(type_: str):
-    if type_ in ("model", "collection"):
+def get_spec_type_from_type(type_: Optional[str]):
+    if type_ in ("model", "collection", "dataset"):
         return type_
     else:
         return "rdf"
