@@ -1,6 +1,6 @@
 """fields to be used in the versioned schemas (may return shared raw nodes on `deserialize`"""
 import datetime
-import distutils.version
+import packaging.version
 import logging
 import pathlib
 import typing
@@ -222,6 +222,34 @@ class String(DocumentedField, marshmallow_fields.String):
     pass
 
 
+class Name(String):
+    def __init__(
+        self,
+        *,
+        validate: typing.Optional[
+            typing.Union[
+                typing.Callable[[typing.Any], typing.Any], typing.Iterable[typing.Callable[[typing.Any], typing.Any]]
+            ]
+        ] = None,
+        **kwargs,
+    ) -> None:
+        if validate is None:
+            validate = []
+
+        if isinstance(validate, typing.Iterable):
+            validate = list(validate)
+        else:
+            validate = [validate]
+
+        validate.append(
+            field_validators.Predicate("__contains__", "/", invert_output=True, error="may not contain '/'")
+        )
+        validate.append(
+            field_validators.Predicate("__contains__", "\\", invert_output=True, error="may not contain '\\'")
+        )
+        super().__init__(validate=validate, **kwargs)
+
+
 class DOI(String):
     pass
 
@@ -327,7 +355,7 @@ class ImportableSource(String):
             source_file_field = Union(
                 [
                     URL(),
-                    RelativeLocalPath(
+                    Path(
                         validate=field_validators.Attribute(
                             "suffix",
                             field_validators.Equal(
@@ -513,7 +541,7 @@ class SHA256(String):
         return value_str
 
 
-class StrictVersion(String):
+class Version(String):
     def _deserialize(
         self,
         value: typing.Any,
@@ -521,7 +549,7 @@ class StrictVersion(String):
         data: typing.Optional[typing.Mapping[str, typing.Any]],
         **kwargs,
     ):
-        return distutils.version.StrictVersion(str(value))
+        return packaging.version.Version(str(value))
 
 
 class URI(String):
